/*==================================================================================================
* Package : S32Z2E2 BIST example source code
* File    : STCU.c
*
*   Copyright 2024 NXP
*   NXP Confidential and Proprietary. This software is owned or controlled by NXP and may only
*   be used strictly in accordance with the applicable license terms. By expressly accepting
*   such terms or by downloading, installing, activating and/or otherwise using the software,
*   you are agreeing that you have read, and that you agree to comply with and are bound by,
*   such license terms. If you do not agree to be bound by the applicable license terms, then
*   you may not retain, install, activate or otherwise use the software.
*
*   THIS SOFTWARE IS PROVIDED BY NXP "AS IS" AND ANY EXPRESSED OR
*   IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
*   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
*   IN NO EVENT SHALL NXP OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
*   INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
*   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
*   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
*   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
*   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
*   IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
*   THE POSSIBILITY OF SUCH DAMAGE.
*
==================================================================================================*/
#include "typedefs.h"
#include "S32Z2.h"
#include "STCU.h"
#include "MMC_GLS_FULL.h"

/*******************************************************************************
FUNCTION        : SetupAndRunSTCU()
PURPOSE         : Configures the STCU for MBIST and LBIST example test
RETURNS NOTES   : none
 ********************************************************************************/
void SetupAndRunSTCU()
{
	int i, concurrent;
	int rtu_mbist_concurrent = 1;

	EnableRTU (RTU0);
	EnableRTU (RTU1);

	CSTCU.RDEN.B.SERD0 = 0;
	CSTCU.RDEN.B.SERD1 = 0;
	CSTCU.RDEN.B.SERD2 = 0;
	CSTCU.RDEN.B.SERD3 = 0;
    while (CSTCU.RDENSTAT.R !=0) ;

//////////////////////////////////////////////////////////////////////////
//MBIST setup
///////////////////////////////////////////////////////////////////////////
	CSTCU.LSCHLVLD.B.MBPLVLD = 1;	//1 to enable MBIST
    mbist_setup_mtr(MBIST_ALGO_BasicChk + MBIST_ALGO_MarchCp) ;	

	CSTCU.LMBPTR[0].R = 0x00000102;	
	CSTCU.LMBPTR[1].R = 0x00000104;	
	CSTCU.LMBPTR[2].R = 0x00000106;	
	CSTCU.LMBPTR[3].R = 0x0000010c;	
	CSTCU.LMBPTR[4].R = 0x0000010e;	
	CSTCU.LMBPTR[5].R = 0x00000100;	
	CSTCU.LMBPTR[6].R = 0x00000101;	
	CSTCU.LMBPTR[7].R = 0x00000103;	
	CSTCU.LMBPTR[8].R = 0x00000105;	
	CSTCU.LMBPTR[9].R = 0x00000107;	
	CSTCU.LMBPTR[10].R = 0x00000108;
	CSTCU.LMBPTR[11].R = 0x00000109;
	CSTCU.LMBPTR[12].R = 0x0000010a;
	CSTCU.LMBPTR[13].R = 0x0000010b;
	CSTCU.LMBPTR[14].R = 0x00000111;
	CSTCU.LMBPTR[15].R = 0x00000013;

	CSTCU.LMBPTR[16].R = 0x0000000f;
	CSTCU.LMBPTR[17].R = 0x80000012;

	CSTCU.RDEN.B.SERD0 = 1;	//SMU
	CSTCU.RDEN.B.SERD1 = 1;	//RTU0
	CSTCU.RDEN.B.SERD2 = 1;	//RTU1

	while (CSTCU.RDENSTAT.R !=7) ;

///////////////////////////////////////////////////////////////////////////////////
//LBIST setup
///////////////////////////////////////////////////////////////////////////////
 	CSTCU.LSCHLVLD.B.LBPLVLD = 1;	//1 to enable LBIST
    lbist_setup_mmc () ;
	concurrent = 1;

	//all LBISTS
	CSTCU.LLBPTR[0].R = 0x00000000;
	CSTCU.LLBPTR[1].R = 0x00000001;
	CSTCU.LLBPTR[2].R = 0x00000002;
	CSTCU.LLBPTR[3].R = 0x00000004;
	CSTCU.LLBPTR[4].R = 0x00000005;
	CSTCU.LLBPTR[5].R = 0x00000006;
	CSTCU.LLBPTR[6].R = 0x00000009;
	CSTCU.LLBPTR[7].R = 0x0000000a;
	CSTCU.LLBPTR[8].R = 0x0000000b;
	CSTCU.LLBPTR[9].R = 0x0000000c;
	CSTCU.LLBPTR[10].R = 0x0000000e;	//RTU0
	CSTCU.LLBPTR[11].R = 0x0000000f;	//RTU0
	CSTCU.LLBPTR[12].R = 0x00000011;	//RTU1
	CSTCU.LLBPTR[13].R = 0x80000012;	//RTU1
	for (i=0;i<20;i++) {
		CSTCU.LLBPTR[i].B.LBCSM = concurrent;
	}
	CSTCU.RDEN.B.SERD0 = 1;	//SMU
	CSTCU.RDEN.B.SERD1 = 1;	//RTU0
	CSTCU.RDEN.B.SERD2 = 1;	//RTU1
	while (CSTCU.RDENSTAT.R !=7) ;

///////////////////////////////////////////////////////////////////////////////////
//General setup
///////////////////////////////////////////////////////////////////////////////

	CSTCU.ERR_STAT.B.RFSF = 0;
	CSTCU.ERR_STAT.B.UFSF = 0;

	CSTCU.STAG.R = 0x000F0F00;
	CSTCU.WDG.R = 0x00500000;

	LSTCU_0.STAG.R = 0x000F0F00;
	LSTCU_1.STAG.R = 0x000F0F00;
	LSTCU_2.STAG.R = 0x000F0F00;
	LSTCU_3.STAG.R = 0x000F0F00;
	LSTCU_4.STAG.R = 0x000F0F00;
	LSTCU_5.STAG.R = 0x000F0F00;
	LSTCU_6.STAG.R = 0x000F0F00;
	LSTCU_7.STAG.R = 0x000F0F00;
	LSTCU_8.STAG.R = 0x000F0F00;
	LSTCU_9.STAG.R = 0x000F0F00;
	LSTCU_10.STAG.R = 0x000F0F00;
	LSTCU_11.STAG.R = 0x000F0F00;
	LSTCU_12.STAG.R = 0x000F0F00;
	LSTCU_14.STAG.R = 0x000F0F00;
	LSTCU_15.STAG.R = 0x000F0F00;
	LSTCU_17.STAG.R = 0x000F0F00;
	LSTCU_15.STAG.R = 0x000F0F00;

	//set up LSTCU
	//using the defaults for all the LSTCU settings except for the RTUs as have 2x MBISTs
	LSTCU_14.MBPTR[0].B.MBEOL=0;
	LSTCU_14.MBPTR[1].B.MBPTR=1;
	LSTCU_15.MBPTR[0].B.MBEOL=0;
	LSTCU_15.MBPTR[1].B.MBPTR=1;
	LSTCU_17.MBPTR[0].B.MBEOL=0;
	LSTCU_17.MBPTR[1].B.MBPTR=1;
	LSTCU_18.MBPTR[0].B.MBEOL=0;
	LSTCU_18.MBPTR[1].B.MBPTR=1;

	LSTCU_14.PH1_DUR.B.PH1DUR = 0x3ff;
	LSTCU_15.PH1_DUR.B.PH1DUR = 0x3ff;
	LSTCU_17.PH1_DUR.B.PH1DUR = 0x3ff;
	LSTCU_18.PH1_DUR.B.PH1DUR = 0x3ff;

	// set concurrency (or not) for the RTU doublers
	LSTCU_14.MBPTR[0].B.MBCSM = rtu_mbist_concurrent;
	LSTCU_15.MBPTR[0].B.MBCSM = rtu_mbist_concurrent;
	LSTCU_17.MBPTR[0].B.MBCSM = rtu_mbist_concurrent;
	LSTCU_18.MBPTR[0].B.MBCSM = rtu_mbist_concurrent;

	////////////////////////////////////////////////////////////////
	////////////////////// RUN STCU  //////////////////////////////
	////////////////////////////////////////////////////////////////
	CSTCU.RUNSWREG.B.RUNSW = 1;

	for (i=0;i<1000;i++) ;
	while (CSTCU.RUNSWREG.B.RUNSW == 1) {	//wait for STCU to complete
	}
}/* end of SetupAndRunSTCU */


void lbist_setup_mmc () {
   lbist_setup_mmc_pl_ddr() ;
   lbist_setup_mmc_pl_dfa() ;
   lbist_setup_mmc_pl_gtm() ;
   lbist_setup_mmc_pl_hse() ;
   lbist_setup_mmc_pl_ne() ;
   lbist_setup_mmc_pl_nw() ;
   lbist_setup_mmc_pl_se() ;
   lbist_setup_mmc_pl_ss_rtu0_core_clust() ;
   lbist_setup_mmc_pl_ss_rtu0_rest() ;
   lbist_setup_mmc_pl_ss_rtu1_core_clust() ;
   lbist_setup_mmc_pl_ss_rtu1_rest() ;
   lbist_setup_mmc_pl_sw() ;
   lbist_setup_mmc_pl_xm8() ;
   lbist_setup_mmc_pl_north() ;
}



void mbist_setup_mtr (uint32_t algo) {
#define	NUM_MB	22
#define ADDR_LSTCU_0	0x4D408000
#define ADDR_LSTCU_1	0x4D208000
#define ADDR_LSTCU_2	0x4D108000
#define ADDR_LSTCU_3	0x4D288000
#define ADDR_LSTCU_4	0x4D308000
#define ADDR_LSTCU_5	0x4D388000
#define ADDR_LSTCU_6	0x4D008000
#define ADDR_LSTCU_7	0x4D209000
#define ADDR_LSTCU_8	0x4D289000
#define ADDR_LSTCU_9	0x4D409000
#define ADDR_LSTCU_10	0x4D109000
#define ADDR_LSTCU_11	0x4D10A000
#define ADDR_LSTCU_12	0x4D309000
#define ADDR_LSTCU_19	0x4D389000
#define ADDR_RTU0_LSTCU_00	0x4C009000
#define ADDR_RTU0_LSTCU_01	0x4C009800
#define ADDR_RTU0_LSTCU_10	0x4C00A000
#define ADDR_RTU0_LSTCU_11	0x4C00A800
#define ADDR_RTU1_LSTCU_00	0x4C809000
#define ADDR_RTU1_LSTCU_01	0x4C809800
#define ADDR_RTU1_LSTCU_10	0x4C80A000
#define ADDR_RTU1_LSTCU_11	0x4C80A800

	uint32_t i, lb_addr[NUM_MB] = {ADDR_LSTCU_0, ADDR_LSTCU_1, ADDR_LSTCU_2, ADDR_LSTCU_3, ADDR_LSTCU_4, ADDR_LSTCU_5, ADDR_LSTCU_6, ADDR_LSTCU_7, ADDR_LSTCU_8, ADDR_LSTCU_9, ADDR_LSTCU_10, ADDR_LSTCU_11, ADDR_LSTCU_12, ADDR_LSTCU_19, ADDR_RTU0_LSTCU_00,ADDR_RTU0_LSTCU_01, ADDR_RTU0_LSTCU_10,ADDR_RTU0_LSTCU_11, ADDR_RTU1_LSTCU_00,ADDR_RTU1_LSTCU_01,ADDR_RTU1_LSTCU_10,ADDR_RTU1_LSTCU_11} ;

	CAST32(REG32(0x4dbee01c)) = 0x80000000;			//required for all MBIST runs

	for (i=0; i<NUM_MB; i++) set_mbist (lb_addr[i], algo);

}



void set_mbist (uint32_t addr, uint32_t algo) {
#define	IND_ADD		0x7f0
#define	IND_DATA	0x7f4
	CAST32(REG32(addr + IND_ADD))  = 0x00000014;
	CAST32(REG32(addr + IND_DATA)) = 0x00000001;	//reset - reset
	CAST32(REG32(addr + IND_ADD))  = 0x0000001c;
	CAST32(REG32(addr + IND_DATA)) = 0x80000000;	//bstart - soft reset
	CAST32(REG32(addr + IND_ADD))  = 0x00000018;
	CAST32(REG32(addr + IND_DATA)) = 0x100000ff;	//choose all same bits, these should cover all
	CAST32(REG32(addr + IND_ADD))  = 0x00000048;
	CAST32(REG32(addr + IND_DATA)) = algo;

	//additional writes to setup custom MBIST for Checker+
	CAST32(REG32(addr + IND_ADD))  = 0x00000028;
	CAST32(REG32(addr + IND_DATA)) = 0x00020000;	//MBIST algo setup
	CAST32(REG32(addr + IND_ADD))  = 0x00000024;
	CAST32(REG32(addr + IND_DATA)) = 0x000002C2;	//MBIST algo setup
	CAST32(REG32(addr + IND_ADD))  = 0x00000040;
	CAST32(REG32(addr + IND_DATA)) = 0x00000002;	//MBIST algo setup
}

void EnableRTU (int id) {
	enable_r52_cluster_fence_drain(id, 0xfff, 0x1, 0x0);
	wait_for_rtu_noc_idle(id);
	enable_rtu_sys_master_fence_drain(id);
	enable_output_isolation(id);
	}

void enable_r52_cluster_fence_drain(int id, vuint32_t fd, vuint32_t slave_fd, vuint32_t iso_en) {
  vuint32_t rdata,i;
  //Enable fencing on all Master port
  if(id == RTU0) {
    rdata = RTU0__GPR.CFG_FD.R;
    RTU0__GPR.CFG_FD.R = ((rdata & 0xfffff000) | (fd & 0xfff));
    for (i=0;i<1000;i++) ;
    if(slave_fd != 0x0) {
      // Enable fencing on slave port
  	rdata = RTU0__GPR.CFG_FD.R;
  	RTU0__GPR.CFG_FD.R = ((rdata & 0xfffeffff) | 0x10000);
	      for (i=0;i<1000;i++) ;
    }
    if(iso_en != 0) {
	    	rdata = RTU0__GPR.CFG_FD.R;
	    	RTU0__GPR.CFG_FD.R = ((rdata & 0x0fffffff) | (iso_en & 0xf0000000));
    }
  }
  if(id == RTU1) {
	      rdata = RTU1__GPR.CFG_FD.R;
	      RTU1__GPR.CFG_FD.R = ((rdata & 0xfffff000) | (fd & 0xfff));
	      for (i=0;i<1000;i++) ;
    if(slave_fd != 0x0) {
      // Enable fencing on slave port
	    	rdata = RTU1__GPR.CFG_FD.R;
	    	RTU1__GPR.CFG_FD.R = ((rdata & 0xfffeffff) | 0x10000);
		      for (i=0;i<1000;i++) ;
    }
    if(iso_en != 0) {
	    	rdata = RTU1__GPR.CFG_FD.R;
	    	RTU1__GPR.CFG_FD.R = ((rdata & 0x0fffffff) | (iso_en & 0xf0000000));
    }
  }
}


void enable_rtu_sys_master_fence_drain(int id) {
	int i;

	  if(id == RTU0) {
		  GPR3.RTU0FDC.B.FDREQ_LLC = 1;
			for (i=0;i<1000;i++) ;
	  }
	  if(id == RTU1) {
		  GPR3.RTU1FDC.B.FDREQ_LLC = 0;
			for (i=0;i<1000;i++) ;
	  }
}

void wait_for_rtu_noc_idle(int id) {
	int i;
	  if(id == RTU0) {
			RDC_0.RD1_CTRL_REG.R= (RDC_0.RD1_CTRL_REG.R | 0x80000000);      // Unlocking the RDC register
			RDC_0.RD1_CTRL_REG.R= (RDC_0.RD1_CTRL_REG.R & 0xFFFFFFF7);      // Writing the bit to enable XBAR interface
			for (i=0;i<1000;i++) ;
	RDC_0.RD1_CTRL_REG.R= (RDC_0.RD1_CTRL_REG.R & 0x7FFFFFFF);              // lock the register
}
	  if(id == RTU1) {
			RDC_1.RD1_CTRL_REG.R= (RDC_1.RD1_CTRL_REG.R | 0x80000000);      // Unlocking the RDC register
			RDC_1.RD1_CTRL_REG.R= (RDC_1.RD1_CTRL_REG.R & 0xFFFFFFF7);      // Writing the bit to enable XBAR interface
			for (i=0;i<1000;i++) ;
	RDC_1.RD1_CTRL_REG.R= (RDC_1.RD1_CTRL_REG.R & 0x7FFFFFFF);              // lock the register
}
}


void enable_output_isolation(int id) {
	int i;
	  if(id == RTU0) {
	   MC_ME.PRTN1_PCONF.R= (MC_ME.PRTN1_PCONF.R | 0x00000004);
	   MC_ME.PRTN1_PUPD.R= (MC_ME.PRTN1_PUPD.R | 0x00000004);               // Triggering the process to disable output safe stating
}
	  if(id == RTU1) {
		   MC_ME.PRTN2_PCONF.R= (MC_ME.PRTN1_PCONF.R | 0x00000004);
		   MC_ME.PRTN2_PUPD.R= (MC_ME.PRTN1_PUPD.R | 0x00000004);           // Triggering the process to disable output safe stating
}
		MC_ME.CTL_KEY.R = 0x5AF0;                                           // Key
		MC_ME.CTL_KEY.R = 0xA50F;                                           // Invert Key to initiate the process
	if(id == RTU0) {
			for (i=0;i<1000;i++) ;
	}
	if(id == RTU1) {
			for (i=0;i<1000;i++) ;
	}
}
